package edu.uoc.pfc.mena.webapp.utils;

import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.Writer;

import org.apache.log4j.Logger;





public class FileUtils {
  private static final Logger log = Logger.getLogger( FileUtils.class );

  private enum FileFrom {
    CLASSLOADER, EXTERNAL_PATH;
  }

  /**
   * Obté un arxiu com Stream al que es té accés des del classloader
   * @param filename Nom  de l'arxiu
   * @return Stream de l'arxiu o null si no ho troba
   */
  public static InputStream getFileAsStreamFromClassLoader(String filename) {
    InputStream is = Thread.currentThread().getContextClassLoader().getResourceAsStream( filename );
    if( is == null ){
      log.warn( "L'arxiu [" + filename + "] no existeix" );
    }
    return is;
  }//getFileAsStreamFromClassLoader

  /**
   * Obté un arxiu com Stream buscant-ho a través del seu path absolut
   * @param path Path absolut de l'arxiu
   * @return Stream de l'arxiu o null si no ho troba
   */
  public static InputStream getFileAsStream(String path) {
    FileInputStream fis = null;

    try{
      fis = new FileInputStream( path );
    }catch ( FileNotFoundException e ){
      log.error( "L'arxiu [" + path + "] no existeix", e );
    }
    return fis;
  }//getFileAsStream

  /**
   * Obté el contingut del Stream en forma de String. 
   * Si es produeix algun error, per exemple perquè el Stream no conté un String es retornarà null
   * @param is Stream
   * @return String amb l'arxiu o null en cas d'error
   * @throws Exception
   */
  public static String getStringContentFromStream(InputStream is) {
    StringBuffer content = null;
    try{

      byte[] byteArrayInput = new byte[4096];
      int bytesLeidos = -1;

      while( ( bytesLeidos = is.read( byteArrayInput ) ) > 0 ){
        if( content == null )
          content = new StringBuffer( new String( byteArrayInput, 0, bytesLeidos ) );
        else
          content = content.append( new String( byteArrayInput, 0, bytesLeidos ) );
      }

    }catch ( Exception ex ){
      log.error( "S'ha produït un error obtenint el contingut del stream en un string", ex );
    }finally{
      try{
        is.close();
      }catch ( Exception ex ){
      }
    }
    return ( content != null ? content.toString() : null );
  }

  
  /**
   * 
   * @param is
   * @return
   */
  public static byte [] getContentFromStream(InputStream is) {
    byte [] content = null;
    try {
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      copy(is, baos);
      baos.flush();
      content =  baos.toByteArray();
    }catch ( Exception ex ){
      log.error( "S'ha produit un error obtenint el contingut del stream", ex );      
    } finally {
      try{
        is.close();
      }catch ( Exception ex ){
      }
    }
    return content;
  }
  
  /**
   * Obté un arxiu de text al que es té accés des del classloader
   * @param textFilename  Nom  de l'arxiu
   * @return String amb el contingut de l'arxiu o null si no ho troba o es produeix un altre tipus d'error
   */
  public static String getTextFileFromClassLoader(String textFilename) {
    return getTextFile( textFilename, FileFrom.CLASSLOADER );
  }//getTextFileFromClassLoader

  /**
   * Obté un arxiu de text buscant-ho a través del seu path absolut
   * @param filePath Path absolut de l'arxiu
   * @return String amb el contingut de l'arxiu o null si no ho troba o es produeix un altre tipus d'error
   */
  public static String getTextFile(String filePath) {
    return getTextFile( filePath, FileFrom.EXTERNAL_PATH );
  }//getTextFile

  
  /**
   * Copia el stream d'entrada en el stream de sortida
   * @param in Stream d'entrada
   * @param out Stream de sortida
   */
  public static void copy(InputStream in, OutputStream out)
  {
    byte[] buffer = new byte[1024];
    int numRead;
    try{
      while ((numRead = in.read(buffer)) != -1)
      {
        out.write(buffer,0,numRead);
      }
    }catch ( IOException e ){
      log.error( "S'ha produït un error copiant el stream", e );
    }
  }//copy
  
  /**
   * Copia el contingut del reader en el writer
   * @param in Reader
   * @param out Writer
   */
  public static void copy(Reader in, Writer out)
  {
    char[] buffer = new  char[1024];
    int numRead;
    try{
      while ((numRead = in.read(buffer)) != -1)
      {
        out.write(buffer,0,numRead);
      }
    }catch ( IOException e ){
      log.error( "S'ha produït un error copiant el contingut del reader", e );
    }
  }//copy
  
  /**
   * Obté un arxiu de text 
   * @param textFilename Nom o Path absolut de l'arxiu, segons from
   * @param from Indica des d'on es buscarà l'arxiu (classloader, path absolut)
   * @return String amb el contingut de l'arxiu o null si no ho troba o es produeix un altre tipus d'error
   */
  private static String getTextFile(String textFilename, FileFrom from) {
    String contentXSL = null;
    log.debug( "getTextFile -- Inicio obtencio de l'arxiu [".concat( textFilename ).concat( "]" ) );

    if( textFilename != null && textFilename.length() > 0 ){
      InputStream is = null;

      switch( from ) {
        case CLASSLOADER:
          is = getFileAsStreamFromClassLoader( textFilename );
          break;
        case EXTERNAL_PATH:
          is = getFileAsStream( textFilename );
          break;
      }

      if( is != null ){
        contentXSL = getStringContentFromStream( is );
      }else{
        log.warn( "No hi ha arxiu" );
      }
    }else{
      log.warn( "No hi ha arxiu" );
    }
    log.debug( "getTextFile -- Fin obtencio de l'arxiu [".concat( textFilename ).concat( "]" ) );
    return ( contentXSL != null ? contentXSL.toString() : null );
  }//getTextFile

}// FileUtils
